"""
Module for litchi waypoint photo settings
"""


class Photo:
    """
    Class representing litchi waypoint photo settings

    Attributes:
        time_interval (float): The interval between photos in seconds
        distance_interval (float): The interval between photos in meters
    """

    def __init__(self, time_interval: float = -1.0, distance_interval: float = -1.0):
        """
        Constructor

        Parameters:
            time_interval (float): The interval between photos in seconds, disable with -1.0
            distance_interval (float): The interval between photos in meters, disable with -1.0
        """
        self.time_interval = time_interval
        self.distance_interval = distance_interval

    def set_time_interval(self, interval: float):
        """
        Setter for time based interval

        Parameters:
            interval (float): The interval between photos in seconds, disable with -1.0
        """
        self.time_interval = interval
        self.distance_interval = 0

    def set_distance_interval(self, interval: float):
        """
        Setter for distance based interval

        Parameters:
            interval (float): The interval between photos in seconds, disable with -1.0
        """
        self.distance_interval = interval
        self.time_interval = 0



"""
Module for litchi waypoint actions
"""
# pylint: disable=import-error
from litchi_wp.enums import ActionType


class Action:
    """
    Class respresenting a litchi waypoint action

    Attributes:
        type (ActionType): The type of the action
        param (int): The parameter of the action. Depends on the actiontype
    """

    def __init__(
            self,
            actiontype: ActionType = ActionType.NO_ACTION,
            param: int = 0
    ):
        """
        Constructor

        Parameters:
            actiontype (ActionType): The type of the action
            param (float): The parameter of the action. Depends on the actiontype

                - Stay For (time in milliseconds),
                - Rotate Aircraft (angle in degrees),
                - Tilt Camera (angle in degrees)
                - Take Photo (set to 0)
                - Start Recording (set to 0)
                - Stop Recording (set to 0)
        """
        self.type: ActionType = actiontype
        self.param = param

    def set_type(self, actiontype: ActionType):
        """
        Setter for the type of the action

        Parameters:
            actiontype (Actiontype): The type of the action
        """
        self.type = actiontype.value

    def set_param(self, param: int):
        """
        Setter for the action parameter

        Parameters:
            param (int): The parameter of the action. Depends on the actiontype

                - Stay For (time in milliseconds),
                - Rotate Aircraft (angle in degrees),
                - Tilt Camera (angle in degrees)
        """
        self.param = param

    def delete(self):
        """
        Clear action
        """
        self.type = ActionType.NO_ACTION
        self.param = 0

    def set_stay_for(self, msec: int):
        """
        Setter for 'Stay For' action

        Parameters:
            msec (int): Time to stay in milliseconds
        """
        self.type = ActionType.STAY_FOR
        self.param = msec

    def set_take_photo(self):
        """
        Setter for 'Take Photo' action
        """
        self.type = ActionType.TAKE_PHOTO
        self.param = 0

    def set_start_rec(self):
        """
        Setter for 'Start Recording' action
        """
        self.type = ActionType.START_RECORDING
        self.param = 0

    def set_stop_rec(self):
        """
        Setter for 'Stop Recording' action
        """
        self.type = ActionType.STOP_RECORDING
        self.param = 0

    def set_rotate(self, deg: int):
        """
        Setter for 'Rotate Aircraft' action

        Parameters:
            deg (int): Rotation in degrees
        """
        self.type = ActionType.ROTATE_AIRCRAFT
        self.param = deg

    def set_tilt_cam(self, deg: int):
        """
        Setter for 'Tilt Camera' action

        Parameters:
            deg (int): Tiltangle in degrees
        """
        self.type = ActionType.TILT_CAMERA
        self.param = deg



"""
Module to bundle all enum classes
"""
from enum import Enum


class RotationDirection(Enum):
    """
    Enum class for rotation direction
    """
    CW = 0
    CCW = 1


class GimbalMode(Enum):
    """
    Enum class for Gimbalmode
    """
    DISABLED = 0
    FOCUS_POI = 1
    INTERPOLATE = 2


class AltitudeMode(Enum):
    """
    Enum class for Altitudemode
    """
    MSL = 0
    AGL = 1


class ActionType(Enum):
    """
    Enum class for action types
    """
    NO_ACTION = -1
    STAY_FOR = 0
    TAKE_PHOTO = 1
    START_RECORDING = 2
    STOP_RECORDING = 3
    ROTATE_AIRCRAFT = 4
    TILT_CAMERA = 5



"""
Module for litchi waypoint POIs
"""
# pylint: disable=import-error
from litchi_wp.enums import AltitudeMode


class Poi:
    """
    Class representing a litchi waypoint POI

    Attributes:
        lat (float): Latitude coordinate for the waypoint in WGS84 format
        lon (float): Longitude coordinate for the waypoint in WGS84 format
        altitude (float): The altitude in meters
        altitude_mode (int): The altitudemode
    """

    def __init__(
            self,
            lat: float = 0,
            lon: float = 0,
            altitude: float = 0,
            altitude_mode: AltitudeMode = AltitudeMode.MSL
    ):
        """
        Constructor

        Parameters:
            lat (float): Latitude coordinate for the waypoint in WGS84 format
            lon (float): Longitude coordinate for the waypoint in WGS84 format
            altitude (float): The altitude in meters
            altitude_mode (AltitudeMode): The altitudemode, MSL or AGL
        """
        self.lat = lat
        self.lon = lon
        self.altitude = altitude
        self.altitude_mode = altitude_mode

    def set_coordinates(self, lat: float, lon: float):
        """
        Setter for coordinates

        Parameters:
        lat (float): Latitude coordinate for the waypoint in WGS84 format
        lon (float): Longitude coordinate for the waypoint in WGS84 format
        """
        self.lat = lat
        self.lon = lon

    def set_altitude(self, altitude: float):
        """
        Setter for altitude

        Parameters:
        altitude (float): The altitude in meters
        """
        self.altitude = altitude

    def set_altitude_mode(self, altitude_mode: AltitudeMode):
        """
        Setter for the altitude mode

        Parameters:
            altitude_mode (AltitudeMode): The altitudemode (MSL or AGL)
        """
        self.altitude_mode = altitude_mode



"""
Module for litchi waypoint altitude settings
"""
# pylint: disable=import-error
from litchi_wp.enums import AltitudeMode


class Altitude:
    """
    Class representing litchi waypoint altitude settings

    Attributes:
        value (float): The height in meters
        mode (AltitudeMode): The altitude mode
    """

    def __init__(self, value: float = -1.0, mode: AltitudeMode = AltitudeMode.AGL):
        """
        Constructor

        value (float): The height in meters
        mode (AltitudeMode): The altitude mode (MSL or AGL)
        """
        self.value = value
        self.mode = mode

    def set_mode(self, mode: AltitudeMode):
        """
        Setter for Altitudemode

        Parameters:
            mode (AltitudeMode): The enum of Altitudemode
        """
        self.mode = mode

    def set_value(self, value: float):
        """
        Setter for value

        Parameters:
            value (float): The height in meters
        """
        self.value = value



"""
Module for litchi waypoint gimbal settings
"""
# pylint: disable=import-error
from litchi_wp.enums import GimbalMode


class Gimbal:
    """
    Class representing litchi waypoint gimbal settings

    Attributes:
            mode (GimbalMode): The mode for the gimbal setting
            pitchangle (int): The gimbal angle in degrees
    """

    def __init__(self, mode: GimbalMode = GimbalMode.DISABLED, pitchangle: float = 0):
        """
        Constructor

            mode (GimbalMode): The mode for the gimbal (disabled, focus poi or interpolate)
            pitchangle (float): The gimbal angle in degrees
        """
        self.mode = mode
        self.pitchangle = pitchangle

    def set_focus_poi(self):
        """
        Setter for focus poi mode
        """
        self.set_mode(GimbalMode.FOCUS_POI)

    def set_interpolate(self, pitchangle: float):
        """
        Setter for interpolate mode
        """
        self.set_mode(GimbalMode.INTERPOLATE)
        self.set_pitchangle(pitchangle)

    def set_mode(self, mode: GimbalMode):
        """
        Setter for mode

        Parameters:
            mode (float): The mode for the gimbal (disabled, focus poi or interpolate)
        """
        self.mode = mode

    def set_pitchangle(self, pitchangle: float):
        """
        Setter for pitchangle

        Parameters:
            pitchangle (float): The gimbal angle in degrees
        """
        self.pitchangle = pitchangle



"""
Module for working with the litchi csv waypoints
"""
# pylint: disable=import-error,too-many-arguments,too-many-instance-attributes
from litchi_wp.action import Action, ActionType
from litchi_wp.altitude import Altitude, AltitudeMode
from litchi_wp.enums import RotationDirection
from litchi_wp.gimbal import Gimbal, GimbalMode
from litchi_wp.photo import Photo
from litchi_wp.poi import Poi


class Waypoint:
    """
    Class representing a litchi waypoint

    Attributes:
        lat (float): Latitude coordinate for the waypoint in WGS84 format
        lon (float): Longitude coordinate for the waypoint in WGS84 format
        altitude (float): Altitude in meters
        heading (float): Heading in degrees
        curvesize (float): Curvesize (radius?) in meter
        rotationdir (RotationDirection):
            Direction of rotation ??(0=cw, 1=ccw)?? (always 0 from mission hub)
        gimbal (Gimbal): Gimbal settings
        actions (list[Action]): List of Action objects
        speed (float): Speed in meters per second
        poi (Poi): Poi object
        photo (Photo): Photo object
    """

    def __init__(
            self,
            lat: float,
            lon: float,
            alt: float,
            head: float = 180,
            curve: float = 0,
            rot: RotationDirection = RotationDirection.CW,
            speed: float = 0,
    ):
        self.lat = lat
        self.lon = lon
        self.altitude = Altitude(value=alt)
        self.heading = head
        self.curvesize = curve
        self.rotationdir = rot
        self.gimbal = Gimbal()
        self.actions = [Action() for i in range(0, 15)]
        self.speed = speed
        self.poi = Poi()
        self.photo = Photo()

    def set_coordinates(self, lat: float, lon: float):
        """
        Setter for coordinates

        Parameters:
        lat (float): Latitude coordinate for the waypoint in WGS84 format
        lon (float): Longitude coordinate for the waypoint in WGS84 format
        """
        self.lat = lat
        self.lon = lon

    def set_altitude(self, value: float, mode: AltitudeMode = AltitudeMode.AGL):
        """
        Setter for altitude

        Parameters:
            value (float): The height in meters
            mode (AltitudeMode): The altitude mode (MSL or AGL)
        """
        self.altitude.set_value(value)
        self.altitude.set_mode(mode)

    def set_heading(self, value: float):
        """
        Setter for heading

        Parameters:
            value (float): The heading in degrees (0 = north, 180 = south)
        """
        self.heading = value

    def set_curvesize(self, value: float):
        """
        Setter for curve size

        Parameters:
            value (float): The curve radius in meters
        """
        self.curvesize = value

    def set_rotation_direction(self, value: RotationDirection = RotationDirection.CW):
        """
        Setter for the rotation direction

        Parameters:
            value (RotationDirection): Clockwise or Counterclockwise rotation
        """
        self.rotationdir = value

    def set_gimbal(self, mode: GimbalMode, pitchangle: float = 0):
        """
        Setter for gimbal mode

        Parameters:
            mode (GimbalMode): The mode setting for the gimbal
            pitchangle (float): The angle for the gimbal (only for interpolate mode)
        """
        match mode:
            case GimbalMode.DISABLED:
                self.gimbal.set_mode(GimbalMode.DISABLED)
            case GimbalMode.FOCUS_POI:
                self.gimbal.set_focus_poi()
            case GimbalMode.INTERPOLATE:
                self.gimbal.set_interpolate(pitchangle)

    def set_speed_ms(self, value: float):
        """
        Setter for speed in meters per second

        Parameters:
            value (float): The speed in meters per second
        """
        self.speed = value

    def set_speed_kmh(self, value: float):
        """
        Setter for speed in kilometers per hour

        Parameters:
            value (float): The speed in kilometers per hour
        """
        self.speed = value / 3.6

    def set_poi(
            self,
            lat: float,
            lon: float,
            alt: float,
            alt_mode: AltitudeMode = AltitudeMode.MSL
    ):
        """
        Setter for point of interest

        Parameters:
            lat (float): Latitude coordinate for the waypoint in WGS84 format
            lon (float): Longitude coordinate for the waypoint in WGS84 format
            alt (float): The altitude in meters
            alt_mode (AltitudeMode): The altitudemode, MSL or AGL
        """
        self.poi.set_coordinates(lat, lon)
        self.poi.set_altitude(alt)
        self.poi.set_altitude_mode(alt_mode)

    def set_photo_interval_time(self, seconds: float):
        """
        Setter for photo time interval

        Parameters:
            seconds (float): The time in seconds between each photo
        """
        self.photo.set_time_interval(seconds)

    def set_photo_interval_distance(self, meters: float):
        """
        Setter for photo distance interval

        Parameters:
            meters (float): The distance in meters between each photo
        """
        self.photo.set_distance_interval(meters)

    def set_action(self, index: int, actiontype: ActionType, param: int = 0):
        """
        Setter for actions

        Parameters:
            index (int): Action slot (0, ... ,14)
            actiontype (ActionType): The type of the action
            param (float): The parameter of the action. Depends on the actiontype

                - Stay For (time in milliseconds),
                - Rotate Aircraft (angle in degrees),
                - Tilt Camera (angle in degrees)
                - Take Photo (set to 0)
                - Start Recording (set to 0)
                - Stop Recording (set to 0)
        """
        if index > 14 or index < 0:
            return
        match actiontype:
            case ActionType.NO_ACTION:
                self.actions[index].delete()
            case ActionType.ROTATE_AIRCRAFT:
                self.actions[index].set_rotate(param)
            case ActionType.STAY_FOR:
                self.actions[index].set_stay_for(param)
            case ActionType.TILT_CAMERA:
                self.actions[index].set_tilt_cam(param)
            case ActionType.TAKE_PHOTO:
                self.actions[index].set_take_photo()
            case ActionType.START_RECORDING:
                self.actions[index].set_start_rec()
            case ActionType.STOP_RECORDING:
                self.actions[index].set_stop_rec()

    @staticmethod
    def get_header(line_break='\n'):
        """
        Getter for the waypoint file header
        """
        ret = 'latitude,longitude,altitude(m),heading(deg),' \
              'curvesize(m),rotationdir,gimbalmode,gimbalpitchangle,' \
              'actiontype1,actionparam1,actiontype2,actionparam2,' \
              'actiontype3,actionparam3,actiontype4,actionparam4,' \
              'actiontype5,actionparam5,actiontype6,actionparam6,' \
              'actiontype7,actionparam7,actiontype8,actionparam8,' \
              'actiontype9,actionparam9,actiontype10,actionparam10,' \
              'actiontype11,actionparam11,actiontype12,' \
              'actionparam12,actiontype13,actionparam13,actiontype14,' \
              'actionparam14,actiontype15,actionparam15,' \
              'altitudemode,speed(m/s),poi_latitude,' \
              'poi_longitude,poi_altitude(m),poi_altitudemode,' \
              'photo_timeinterval,photo_distinterval'
        if line_break:
            ret += line_break
        return ret

    def to_line(self, line_break='\n'):
        """
        Transforms the waypoint to a line in litchi csv format
        """
        line = ''
        line += str(self.lat) + ','
        line += str(self.lon) + ','
        line += str(self.altitude.value) + ','
        line += str(self.heading) + ','
        line += str(self.curvesize) + ','
        line += str(self.rotationdir.value) + ','
        line += str(self.gimbal.mode.value) + ','
        line += str(self.gimbal.pitchangle) + ','
        for i in range(0, 15):
            line += str(self.actions[i].type.value) + ','
            line += str(self.actions[i].param) + ','
        line += str(self.altitude.mode.value) + ','
        line += str(self.speed) + ','
        line += str(self.poi.lat) + ','
        line += str(self.poi.lon) + ','
        line += str(self.poi.altitude) + ','
        line += str(self.poi.altitude_mode.value) + ','
        line += str(self.photo.time_interval) + ','
        line += str(self.photo.distance_interval)
        if line_break:
            line += line_break
        return line
